/*
  Java gfx-test framework

   Copyright (C) 2010, 2011  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.harness;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;

import javax.swing.JLabel;
import javax.swing.JPanel;



/**
 * Class representing panel which displays value of selected pixel from sample
 * image and value of relevant pixel from the test image. Pixel values are
 * printed as hexadecimal triplets (as in HTML) + colored panel (it's background
 * is changed). Additionally red, green and blue difference between the two
 * pixels are computed and shown on this panel too.
 * 
 * @author Pavel Tisnovsky
 */
public class PixelPanel extends JPanel
{
    /**
     * Generated serial version UID.
     */
    private static final long serialVersionUID = 579405820041982604L;

    /**
     * Label containing fixed test.
     */
    private JLabel samplePixelLabel = new JLabel("Sample:");

    /**
     * Label containing fixed test.
     */
    private JLabel testedPixelLabel = new JLabel("Tested:");

    /**
     * Label containing fixed test.
     */
    private JLabel differenceLabel = new JLabel("Diff:");

    /**
     * Label containing first pixel value (color)
     */
    private JLabel samplePixelValue = new JLabel("?");

    /**
     * Label containing second pixel value (color)
     */
    private JLabel testedPixelValue = new JLabel("?");
    
    /**
     * Label containing difference between two pixels.
     */
    private JLabel differenceValue = new JLabel("?");
    
    /**
     * Panel used to shown color of selected pixel from sample image.
     */
    private JPanel samplePixelPanel = new JPanel();
    
    /**
     * Panel used to shown color of selected pixel from tested image.
     */
    private JPanel testedPixelPanel = new JPanel();

    /**
     * Constructor.
     */
    public PixelPanel()
    {
        super();
        setComponentSizes();
        setLayout();
        addComponentsToPanel();
    }
    
    /**
     * Set sizes of panels using to shown pixel color.
     */
    private void setComponentSizes()
    {
        setSize(this.samplePixelPanel);
        setSize(this.testedPixelPanel);
    }

    /**
     * Set the panel layout.
     */
    private void setLayout()
    {
        this.setLayout(new GridLayout(3, 3, 0, 5));
    }

    /**
     * Add all components to panel.
     */
    private void addComponentsToPanel()
    {
        this.add(this.samplePixelLabel);
        this.add(this.samplePixelValue);
        this.add(this.samplePixelPanel);
        this.add(this.testedPixelLabel);
        this.add(this.testedPixelValue);
        this.add(this.testedPixelPanel);
        this.add(this.differenceLabel);
        this.add(this.differenceValue);
    }
    
    /**
     * Set size of one panel used to show pixel color.
     * 
     * @param panel
     *            selected panel
     */
    private void setSize(JPanel panel)
    {
        Dimension dimension = new Dimension(20, 20);
        panel.setMaximumSize(dimension);
        panel.setMinimumSize(dimension);
        panel.setPreferredSize(dimension);
    }

    /**
     * Compute and show value of pixel from sample image.
     * 
     * @param rgb
     *            pixel value, ie. its RGB+alpha (unused)
     */
    private void computeAndShowSamplePixelValue(int rgb)
    {
        this.samplePixelValue.setText(String.format("%06x", Integer.valueOf(rgb)));
        this.samplePixelPanel.setBackground(new Color(rgb));
    }

    /**
     * Compute and show value of pixel from tested image.
     * 
     * @param rgb
     *            pixel value, ie. its RGB+alpha (unused)
     */
    private void computeAndShowTestedPixelValue(int rgb)
    {
        this.testedPixelValue.setText(String.format("%06x", Integer.valueOf(rgb)));
        this.testedPixelPanel.setBackground(new Color(rgb));
    }
    
    /**
     * Compute and show differences between two pixels - one pixel is from
     * sample image, second pixel is from tested image.
     * 
     * @param color1 first pixel color
     * @param color2 second pixel color
     */
    private void computeAndShowPixelDifferenceValue(Color color1, Color color2)
    {
        int deltaRed = color1.getRed() - color2.getRed();
        int deltaGreen = color1.getGreen() - color2.getGreen();
        int deltaBlue = color1.getBlue() - color2.getBlue();
        this.differenceValue.setText(String.format("(%d) (%d) (%d)", Integer.valueOf(deltaRed), Integer.valueOf(deltaGreen), Integer.valueOf(deltaBlue)));
    }

    /**
     * Show pixel values after the pixel(s) are selected by mouse.
     *
     * @param color1 first pixel color
     * @param color2 second pixel color
     */
    public void showPixelValues(Color color1, Color color2)
    {
        this.computeAndShowSamplePixelValue(color1.getRGB());
        this.computeAndShowTestedPixelValue(color2.getRGB());
        this.computeAndShowPixelDifferenceValue(color1, color2);
    }
}
